% Compute the proportional-damping matrices of the individual beam gcells.
% Return an element set of them so they may be assembled.
%   
% function ems = damping_proportional(self, geom0, u1, Rfield1, dchi)
% 
%     geom0=initial geometry field; locations of nodes
%     u1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     dchi=field of translation and rotation increments
%             Equation numbers are taken from the field dchi.   
%
% The damping matrices are computed in the local coordinate system and
% then transformed into the global coordinate system. 
%
function ems = damping_proportional(self, geom0, u1, Rfield1, dchi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom0,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Ces = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(dchi,conns(i,:),'eqnums');
        Ces{i} =zeros(6*nfens);
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    Rs = gather(Rfield1, (1:get(geom0,'nfens')), 'values', 'noreshape'); % displacement in step n
    X1s=Xs+U1s;
   % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );% Young's modulus
    G=get(prop , 'G' );% shear modulus
    rho=get(prop , 'rho' );
    % Section properties
    A=get(gcells,'A');
    I1=get(gcells,'I1');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    J=get(gcells,'J');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % initial coordinates of nodes
        x1 = X1s(conn,1:3); % current coordinates of nodes
        RI =reshape(Rs(conn(1),:),3,3);% current directors at node 1
        RJ =reshape(Rs(conn(2),:),3,3);% current directors at node 2
        [L1,T,d_N]=local_frames(x0,x1x2_vector(:,i),x1,RI,RJ);
        if (self.Rayleigh_damping_stiffness~= 0)
            Ces{i}=Ces{i}+self.Rayleigh_damping_stiffness*local_stiffness(self, E, G, A(i), I2(i), I3(i), J(i), L1);
        end
        if (self.Rayleigh_damping_mass~= 0)
            L0 =norm((x0(2,:)-x0(1,:)));
            Ces{i}=Ces{i}+self.Rayleigh_damping_mass*local_mass(self, A(i), I1(i), I2(i), I3(i), rho, L0);
        end
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        Ces{i} = Te*Ces{i}*Te';
    end
    ems  = elematset(struct('mat',{Ces}, 'eqnums',{eqnums}));
end