%%  Simply-supported/clamped beam modal analysis. Corotational beam
% 
% Vibration analysis of beam which is simply supported in one plane, and clamped in another. 
% The results are compared with analytical expressions.
% 
% For  the parameters below one should get
%   Eigenvector 1 frequency 21.8251 (analyt=21.8254) [Hz]
%   Eigenvector 2 frequency 49.4702 (analyt=49.4758) [Hz] 
%
function CB_beam_modal
[uSEC,uMIN,uHR,uDAY,uWK,uMONTH,uYR,uFT,uSLUG,uRAN,...
    uM,uKG,uK,uCM,uGM,uDEG,uRAD,uREV,...
    uIN,uNMI,uYD,uMM,uMILE,uG,uLB,uNT,uOZ,uLBM,uKT,...
    uMPH,uPSI,uPA,uBAR,uATM,uTORR,ummHG,uJ,...
    uCAL,uMEV,uERG,uBTU,uW,uMW,uHP,uA,uV,...
    uHZ,uRPS,uRPM,uL,uGAL,uGPM,...
    uMICRO,uKILO,uMILLI,uMEGA,uGIGA,uTERA]=physical_units;
%% 
% Parameters:
E=30002*1000*uPSI;%30002ksi
nu=0.;
b=18*uIN; h=18*uIN; L=300*uIN;% in, cross-section and length
rho=0.2*uLBM/uIN^3;% Pound/g mass density per unit volume

%% 
% Cross-sectional properties
A=b*h;%in^2
I2=b*h^3/12;%cm^4 
I3=b^3*h/12;%cm^4 

%% 
% Choose the mass formulation:
consistent_no_rotation_inertia=0;
consistent_with_rotation_inertia=1;
lumped_diagonal_no_rotation_inertia=2;
lumped_diagonal_with_rotation_inertia=3;
mass_type=2;

%% 
% Analytical frequencies 
% 
% The analytical frequencies were taken from table 8-1 of Formulas for
% natural frequency and mode shape, Robert D. Blevins, Krieger publishing
% company, Malabar Florida, reprint edition 2001.
% 
% The beam has cylindrical supports at either end. Therefore in the
% vertical plane it behaves as a simply supported beam, while in the
% horizontal plane it behaves as a clamped beam.
% 
% Vertical plane, simply supported:
analyt_Frequencies(1)=(1*pi)^2/(2*pi*L^2)*sqrt(E*I2/rho/A);
% Horizontal plane, clamped:
analyt_Frequencies(2)=(4.73004074)^2/(2*pi*L^2)*sqrt(E*I2/rho/A);
neigvs=length(analyt_Frequencies);



%% 
% Select the number of elements per half the length.
n=4;
Parameters=struct ('A',A,'I2',I2,'I3',I3,'J',I2+I3,'I1',I2+I3,...
    'x1x2_vector',[1 0 0], 'label',1);
[fens,gcells] = Beam_member([[0 -L/2 0];[0 L/2 0]],2*n,@gcellset_beam3,Parameters);
tolerance=L/n/100;

%% 
% Material properties
prop = property_linel_iso (struct('E',E,'nu',nu,'rho',rho));
mater = mater_defor_ss_linel_triax (struct('property',prop));
%% 
% Finite element block
feb = feblock_defor_ss_corot_beam(struct ('mater',mater, 'gcells',gcells,'mass_type',mass_type));


%%
% Construct the requisite fields, geometry and displacement
% Initialize configuration variables
geom0 = field(struct ('name',['geom0'], 'dim', 3, 'fens',fens));
u0=0*geom0;
Rfield = field(struct ('name',['Rfield'], 'dim', 9, 'data', ones(get(geom0,'nfens'),1)*reshape(eye(3),1,9)));
Rfield0=Rfield;
% Define the displacement increment field
dchi  = field(struct ('name',['dchi'], 'dim', 6, 'data', zeros(get(geom0,'nfens'),6)));

% Apply EBC's
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 -L/2 0]),'inflate', tolerance));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5,6];% cylindrical
ebc_val=0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
ebc_fenids=fenode_select(fens, struct('box', update_box([],[0 L/2 0]),'inflate', tolerance));
ebc_prescribed=[1];
ebc_comp=[1,2,3,5,6];% cylindrical
ebc_val=ebc_fenids*0;
dchi  = set_ebc(dchi, ebc_fenids, ebc_prescribed, ebc_comp, ebc_val);
dchi  = apply_ebc (dchi);

% Number equations
dchi  = numbereqns (dchi);
neqns=get(dchi, 'neqns');

%% 
% Assemble the global discrete system
K = start (sparse_sysmat, neqns);
K = assemble (K, cat(2,stiffness(feb,geom0,u0,Rfield0,dchi)));
M = start (sparse_sysmat, neqns);
M = assemble (M, mass(feb,geom0,u0,Rfield0,dchi));
Km=get(K, 'mat');
Mm=get(M, 'mat');



%% 
% Solve the eigenvalue problem
options.tol =1e-26;
options.maxit = 5000;
options.disp = 0;
[W,Omega]=eigs(Km,Mm, neigvs,'sm', options);
[Omegas2,ix]=sort((diag(Omega))) ;
Omegas2=somel(diag(Omega),ix);
Frequencies= sqrt(Omegas2)/2/pi;
  
%
for i=1:neigvs
    disp(['  Eigenvector ' num2str(i) ' frequency ' num2str(Frequencies(i)) ' (analyt=' num2str(analyt_Frequencies(i)) ') [Hz]']);
end
disp(' ')


gv=graphic_viewer;
gv=reset (gv,[]);
scale=50;
for i=1:neigvs
    dchi = scatter_sysvec(dchi, W(:,ix(i)));
    transl=slice(dchi,(1:3),'transl');
    rotat=slice(dchi,(4:6),'rotat');
    wv = get(transl,'values');
    wmag = sqrt(wv(:,1).^2+wv(:,2).^2+wv(:,3).^2);
    dcm=data_colormap(struct ('range',[min(wmag),max(wmag)], 'colormap',jet));
    colors=map_data(dcm, wmag);
    colorfield = field(struct ('name',['cf'], 'dim', 3, 'data',colors));
    reset (gv,struct('limits',[-L/4 L/4 -L/2 L/2 -L/4 L/4]));
    draw(feb, gv, struct ('x', geom0, 'u', 0*transl, 'rot', 0*rotat, 'facecolor','none'));
    axis tight
    h=annotation('textbox',[0.05, 0.05, 0.4, 0.2],...
        'string',['Eigenvector ' num2str(i)  ', Frequency ' num2str(Frequencies(i))]);
    set(h,'fontname','Times','fontsize',16 );
    for xscale=scale*sin((0:1:42)/21*2*pi)
        reset (gv,struct('limits',[-L/4 L/4 -L/2 L/2 -L/4 L/4]));
        camset(gv,1.0e+003 *[-1.3592   -1.7850    1.3095    0.0105         0    0.0105         0         0    0.0010    0.0037]);
        draw(feb, gv, struct ('x', geom0, 'u', 0*transl, 'rot', 0*rotat, 'facecolor','none'));
        draw(feb, gv, struct ('x', geom0, 'u', xscale*transl, 'rot', xscale*rotat,'colorfield',colorfield));
        draw_axes (gv,struct('length',L/10 )); axis off
        pause(0.005);
    end
    delete(h);
end
close(gcf);

assignin('caller','faesor_test_passed',((norm(analyt_Frequencies-Frequencies'))<1e-2));
end