function [t,g,p,U,E] = VLV(h,hc,T,II,wi,Gi,alpha,Gm)
%VLV.m
%
%--------------------------------------------------------------------------
% Implements Lie-Verlet algorithm proposed in  
%
% Bou-Rabee, N. and Marsden, J. E. [2008].
% Hamilton-Pontryagin Integrators on Lie Groups.
% FOUNDATIONS OF COMPUTATIONAL MATHEMATICS.
% 9: 197-219.
%
%... Last Modified: May 27, 2010
%... Authors:       Nawaf Bou-Rabee (nawaf@cims.nyu.edu)
%                   Giulia Ortolan (ortolang@dei.unipd.it)
%                   Alessandro Saccon (asaccon@isr.ist.utl.pt)
%--------------------------------------------------------------------------
%
%==========================================================================
%h		= the requested time-stepsize for the simulation;
%hc     = coarse time-stepsize we are considering in order to save data;
%T      = total length of the simulation;
%II     = inertia vector;
%wi     = initial body angular velocity (rotating coordinates);
%Gi     = initial configuration;
%alpha  = strength of magnetic field;
%==========================================================================

% Setup
N = floor(T/h)+1;   % # of simulation steps
R = floor(hc/h);    % # of simulation steps per coarse step
NC= floor(N/R);     % # of coarse steps

maxiter=10;         % maximum # of Newton iterations

% Allocate space
t=zeros(NC,1);   % coarse time mesh
g=zeros(NC,3,3); % orientation of body
p=zeros(NC,3);   % body angular momentum
E=zeros(NC,1);   % total energy
U=zeros(NC,1);   % potential energy

% Initialize algorithm
w0=wi;          % body angular velocity at time 0
p0=II.*w0;      % body angular momentum at time 0
G0=Gi;          % configuration at time 0

% Saving data
U(1)=potential(G0,alpha,Gm); % potential energy at time 0
E(1)=0.5*p0'*w0+U(1); % total energy at time 0
p(1,:)=p0;      % saving data
g(1,:,:)=G0;    % saving data

% Working variables
Jac=zeros(3,3);  % jacobian
PxW=zeros(3,1);  % cross product pi x omega
ii=1; % bookmark for the coarse time grid

tau=potentialtorque(G0,alpha,Gm); % potential torque applied at time 0
J=diag(II);         % inertia matrix

%.........................................................................%
for i=2:N  %... VLV iteration rule .......................................%
%.........................................................................%
    
    %... Step 1) Newton Solve ............................................%
    
    P1=p0; % initialization
    W1=w0; % initialization
    
    for j=1:maxiter
        F=(W1.'*P1); 
        PxW = 0.5*h*[P1(2)*W1(3)-P1(3)*W1(2);
                     P1(3)*W1(1)-P1(1)*W1(3);
                     P1(1)*W1(2)-W1(1)*P1(2)]; % 0.5*h*(P1 x W1)
        
        f = p0-P1+PxW-0.25*h^2*F*W1+0.5*h*tau; 

        if (norm(f)<1e-10)
            break;
        elseif j==maxiter
            disp('maximum # of iterations exceeded');
        end

        Jac=-J+0.5*h*([0 -P1(3)+II(2)*W1(3) P1(2)-II(3)*W1(2);
            P1(3)-II(1)*W1(3)  0   -P1(1)+II(3)*W1(1); 
            -P1(2)+II(1)*W1(2)  P1(1)-II(2)*W1(1)  0]-h*W1*P1')...
            -0.25*h^2*F*eye(3);

        W1 = W1 - Jac\f;
        P1=II.*W1;
    end
    
    %... Step 2) Explicit Configuration Update ...........................%
    
    x=h*W1;
    theta=norm(x);
    c1=4.0/(4.0+theta^2); c2=0.5*c1; %Cayley map
    % c1=sin(theta)/theta; c2=(1.0-cos(theta))/(theta^2); % exponential map
    G1=G0*...
    [1.0+c2*(x(1)^2-theta^2) c2*x(1)*x(2)-c1*x(3)  c2*x(1)*x(3)+c1*x(2);
    c2*x(1)*x(2)+c1*x(3)   1.0+c2*(x(2)^2-theta^2) c2*x(2)*x(3)-c1*x(1);
    c2*x(1)*x(3)-c1*x(2)     c2*x(2)*x(3)+c1*x(1) 1.0+c2*(x(3)^2-theta^2)]; 
                                                                                
            
    %... Step 3) Explicit Momentum Update ................................%
    
    %... Note: only one force evaluation per step
    tau=potentialtorque(G1,alpha,Gm); % potential torque applied at time k+1
    
    p1 = P1+PxW+0.25*h^2*(W1'*P1)*W1+0.5*h*tau; % body angular momentum
    w1 = p1./II;                                % body angular velocity

    %... Update Statistics
     
    if (mod(i,R) == 0) 
        ii=ii+1; % bookmark for the coarse time grid
        t(ii)=t(ii-1)+R*h; % coarse time mesh
        
        U(ii) = potential(G1,alpha,Gm); % potential energy
        E(ii) = 0.5*w1'*p1+U(ii);       % total energy
        p(ii,:) = p1; 
        g(ii,:,:) = G1;
    
    end
    
    %... Iterate
    
    p0=p1;
    w0=w1;
    G0=G1;

%.........................................................................%
end        %... VLV iteration rule .......................................%
%.........................................................................%
end


%-------------------------
function V=potential(G,alpha,Gm)
%Potential Energy Subfunction.

    dm=sqrt(2*(3-sum(diag(Gm'*G))));
    de=sqrt(2*(3-sum(diag(G))));
   
    V=(de-1.0)^2-alpha/dm; 
end
%-------------------------


function tau=potentialtorque(G,alpha,Gm)
%Left Trivialized Potential Torque Subfunction.
    
    dm=sqrt(2*(3-sum(diag(Gm'*G))));
    de=sqrt(2*(3-sum(diag(G))));
    
    tau=2.0*(1.0-1.0/de)*[G(2,3) - G(3,2);
                          G(3,1) - G(1,3);
                          G(1,2) - G(2,1)] ...
       + alpha*1.0/dm^3*[G(1,3)*Gm(1,2) - G(1,2)*Gm(1,3) - G(2,2)*Gm(2,3) + G(2,3)*Gm(2,2) - G(3,2)*Gm(3,3) + G(3,3)*Gm(3,2); ...
                         G(1,1)*Gm(1,3) - G(1,3)*Gm(1,1) + G(2,1)*Gm(2,3) - G(2,3)*Gm(2,1) + G(3,1)*Gm(3,3) - G(3,3)*Gm(3,1); ...
                         G(1,2)*Gm(1,1) - G(1,1)*Gm(1,2) - G(2,1)*Gm(2,2) + G(2,2)*Gm(2,1) - G(3,1)*Gm(3,2) + G(3,2)*Gm(3,1)];
end