%--------------------------------------------------------------------------
% Script to launch Lie integrators to perform long time simulation of a 
% a rigid body in a static spring-magnetic potential field.  This
% numerical test of symplecticity is detailed in:
%
% Bou-Rabee, N., Ortolan, G., and Saccon, A. [2010].
% A counterexample showing the semi-explicit Lie-Newmark algorithm
% is not variational. 
% (preprint).
%
% Results show the following:
%   (i) VLV algorithm exhibits no energy drift;
%   (ii) Lie-Newmark algorithm exhibits energy drift; and,
%   (iii) `explicit' Lie-Midpoint algorithm exhibits energy drift.
%
%... Last Modified: May 27, 2010
%... Authors:       Nawaf Bou-Rabee (nawaf@cims.nyu.edu)
%                   Giulia Ortolan (ortolang@dei.unipd.it)
%                   Alessandro Saccon (asaccon@isr.ist.utl.pt)
%--------------------------------------------------------------------------


clear all; close all; clc;
format long g;

%--------------------------------------------------------------------------
%... parameters, initial conditions
%--------------------------------------------------------------------------


Ii=2.0; Ij=2.0; Ik=4.0;   % inertia parameters

II=[Ii; Ij; Ik]; % diagonal of the inertia matrix

h=0.125;   % h=0.25; % timestep 
hc=0.5;    % coarse timestep
T=15000;   % total simulation time

alpha = 0.3; % magnetic potential 

vm=[1;0.0;1]; vm=vm/norm(vm);
Gm=expm(2.5*wedge(vm)); % attraction point

Wi0=[0.0;0.0;0.625];    % initial body angular velocity     
v1=[0.0;0.7227;0.0];    
Gi0= expm(wedge(v1));   % initial rotation matrix

%--------------------------------------------------------------------------
% Simulate Lie-Newmark 
%--------------------------------------------------------------------------

tic
[ttNMB,gNMB,pNMB,UNMB,HNMB] = NMB(h,hc,T,II,Wi0,Gi0,alpha,Gm);
toc
figure(1); 
plot(ttNMB,HNMB-HNMB(1),'b');


%--------------------------------------------------------------------------
% Simulate `explicit' Lie-Midpoint 
%--------------------------------------------------------------------------

tic
[ttLME,gLME,pLME,ULME,HLME] = LIEMIDEA(h,hc,T,II,Wi0,Gi0,alpha,Gm);
toc
figure(1); hold on;
plot(ttLME,HLME-HLME(1),'g');

%--------------------------------------------------------------------------
% Simulate Lie-Verlet
%--------------------------------------------------------------------------

tic
[ttVLV,gVLV,pVLV,UVLV,HVLV] = VLV(h,hc,T,II,Wi0,Gi0,alpha,Gm);
toc
figure(1); hold on;
plot(ttVLV,HVLV-HVLV(1),'r');

return;

%--------------------------------------------------------------------------
% That's all folks !!
%--------------------------------------------------------------------------

